/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef BITGTKBUFFERLIST_H
#define BITGTKBUFFERLIST_H

#include <gtkmm.h>
#include <map>

#include <bit/recordbuffer.h>

namespace bitGtk {

  /**
  @author Rick L. Vinyard, Jr.
  */
  class BufferList : public Gtk::EventBox
    {
    public:
      BufferList();

      ~BufferList();

      bit::RecordBuffer& buffer();

      void set_buffer( bit::RecordBuffer& buffer);

      void enable_tooltips(bool b=true);

    protected:
      bit::RecordBuffer m_buffer;
      Gtk::Table* m_table;
      int m_mode;
      std::map< bit::FieldBase::pointer, Gtk::Label*> m_value_label;
      unsigned int m_value_label_left;
      Gtk::Tooltips m_tooltips;

      /** Connected to the record changed signal */
      Glib::Dispatcher m_record_changed_dispatcher;

      /** Connected to the field changed signal */
      Glib::Dispatcher m_field_changed_dispatcher;

      /** Locked when the record is being updated */
      Glib::StaticMutex m_record_mutex;

      /** Locked when the field queue is accessed */
      Glib::StaticMutex m_field_queue_mutex;

      /** Queue of fields that need to be updated */
      std::queue<bit::FieldBuffer> m_field_queue;

      /** Callback when the record changes; emits the dispatcher signal. */
      void on_record_changed();

      /** Updates the field when the record changes from the dispatch signal */
      void on_record_changed_dispatcher();

      void on_field_value_changed( bit::FieldBuffer fieldbuffer );

      void on_field_value_changed_dispatcher();

      void set_field_value( bit::FieldBase::pointer field );

      void load_all_field_values();

      void load_all_field_values( bit::FieldBase::pointer );

      void create_table();

      void setup_table();

      unsigned int add_field_label(bit::FieldBase::pointer fb, unsigned int row=0, unsigned int depth=0);

      unsigned int add_field_value_label(bit::FieldBase::pointer fb, unsigned int row=0);

    };

}

#endif
