/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef BIT_GTKBUFFERTABLE_H
#define BIT_GTKBUFFERTABLE_H

#include <vector>
#include <map>
#include <queue>

#include <gtkmm.h>

#include <bit.h>

namespace bitGtk
  {

  /**
  @author Rick L. Vinyard, Jr.
  */
  class BufferTable : public Gtk::HBox
    {
    public:
      BufferTable();

      ~BufferTable();

      bit::FieldBase::pointer record();

      void set_record(bit::FieldBase::pointer record);

      bit::RecordBuffer& buffer();

      void set_buffer( bit::RecordBuffer& buffer);

      void set_bit_width(size_t w);

      void enable_tooltips(bool b=true);

      void enable_highlighter(bool b=true);
      void set_bg_color(std::string s);

      void add_color_set(std::string normal_color, std::string highlight_color="yellow");
      void clear_color_sets();

    protected:
      struct FieldBox
        {
          FieldBox() { }
          std::vector<Gtk::Label*> m_field_labels;
          std::vector<Gtk::EventBox*> m_field_event_boxes;
          std::vector<Gtk::Label*> m_value_labels;
          std::vector<Gtk::EventBox*> m_value_event_boxes;
          std::vector<Gtk::VBox*> m_highlight_boxes;
        };

      std::map<bit::FieldBase::pointer, FieldBox > m_fieldbox;

      bit::RecordBuffer m_buffer;
      bit::FieldBase::pointer m_record;
      int m_mode;
      std::list<std::pair<Gdk::Color, Gdk::Color> > m_colors;
      Gtk::Tooltips m_tooltips;
      bool m_highlighter_enabled;
      Gdk::Color m_bg_color;

      size_t m_display_bit_width;
      Gtk::Table* m_table;
      Gtk::EventBox m_tablebox;
      Gtk::EventBox m_tableboxouter;

      /** Connected to the record changed signal */
      Glib::Dispatcher m_record_changed_dispatcher;

      /** Connected to the field changed signal */
      Glib::Dispatcher m_field_changed_dispatcher;

      /** Locked when the record is being updated */
      Glib::StaticMutex m_record_mutex;

      /** Locked when the field queue is accessed */
      Glib::StaticMutex m_field_queue_mutex;

      /** Queue of fields that need to be updated */
      std::queue<bit::FieldBuffer> m_field_queue;

      /** Callback when the record changes; emits the dispatcher signal. */
      void on_record_changed();

      /** Updates the field when the record changes from the dispatch signal */
      void on_record_changed_dispatcher();

      void on_field_value_changed(bit::FieldBuffer);

      void on_field_value_changed_dispatcher();

      void set_field_value( bit::FieldBase::pointer field );
      void load_all_field_values();
      void load_all_field_values( bit::FieldBase::pointer );

      void create_table();
      void setup_table();
      void create_fieldboxes(bit::FieldBase::pointer fb);
      void add_fieldbox(bit::FieldBase::pointer, unsigned left, unsigned right, unsigned top, unsigned bottom, Gdk::Color bg, Gdk::Color hl);

      bool on_enter(GdkEventCrossing*, bit::FieldBase::pointer);
      bool on_leave(GdkEventCrossing*, bit::FieldBase::pointer);

    };

}

#endif
