/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef BITEXCEPTION_H
#define BITEXCEPTION_H

/**
 * @defgroup exceptions "Exceptions"
 */

#include <stdexcept>

namespace bit
  {

  /**
   * Base struct for all bit library exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  struct bit_exception: public std::runtime_error
    {
      bit_exception(const std::string s="bit: Unknown exception."):
          std::runtime_error(s),
          m_exception_string(s)
      {}

      ~bit_exception() throw () { }

      virtual const char * what () const throw () { return m_exception_string.c_str(); }

    protected:
      std::string m_exception_string;
    };

  namespace exception
    {

    /**
    * @author Rick L Vinyard Jr
    * @ingroup exceptions
     */
    struct invalid_iterator: public bit_exception
    {
      invalid_iterator(): bit_exception("bit: Attempted to use invalid iterator"){ }
    };

    /**
     * @author Rick L Vinyard Jr
     * @ingroup exceptions
     */
    struct invalid_container_op: public bit_exception
    {
      invalid_container_op(): bit_exception("bit: Using container methods on a FieldBase descendant that is not valid for that descendant.") { }
    };

    /**
    * @author Rick L Vinyard Jr
    * @ingroup exceptions
     */
    struct name: public bit_exception
      {
        name(): bit_exception("bit: Name identifiers may not contain the following characters: .[]") { }
        name(const std::string s): bit_exception(s)
        { m_exception_string = "bit: The name \"" + s + "\" is invalid. Name identifiers may not contain the following characters: .[]"; }
      };

      /**
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      struct no_record: public bit_exception
      {
        no_record(): bit_exception("bit:RecordBuffer: Attempted an operation that requires a Record to be associated with this RecordBuffer.") { }
      };

    /**
     * @author Rick L Vinyard Jr
     * @ingroup exceptions
     */
    struct no_recordbuffer: public bit_exception
      {
        no_recordbuffer(): bit_exception("bit:FieldBuffer: Attempted an operation that requires a RecordBuffer to be associated with this FieldBuffer.") { }
      };

    /**
     * @author Rick L Vinyard Jr
     * @ingroup exceptions
     */
    struct invalid_index: public bit_exception
    {
        invalid_index(): bit_exception("bit: Invalid index.") { }
    };

    }

}

#include <bit/except_field.h>
#include <bit/except_record.h>
#include <bit/except_indices.h>
#include <bit/except_type.h>

#endif

