/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef BITRECORD_EXCEPTION_H
#define BITRECORD_EXCEPTION_H

#include <sstream>

#include <bit/except.h>

namespace bit
  {

  /**
   * Base struct for all index exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  struct record_exception: public bit_exception
    {
      record_exception(): bit_exception("bit:Index: Unknown exception.")
      { }
      record_exception(const std::string s): bit_exception(s)
      { }
    }
  ;

  namespace exception
    {


    namespace record
      {

      /**
       * Invalid numeric index identifier
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      struct invalid_numeric_id: public record_exception
        {
          invalid_numeric_id(): record_exception("bit:Index: Invalid numeric identifier, element not found.")
          { }
          invalid_numeric_id(unsigned int id): record_exception("bit:Index: Invalid numeric identifier, element not found.")
          {
            std::ostringstream sout;
            sout << "bit:Index: No element with numeric identifier (" << id << ")";
            m_exception_string = sout.str();
          }
        }
      ;

      /**
       * Invalid string index identifier
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      struct invalid_string_id: public record_exception
        {
          invalid_string_id(): record_exception("bit:Index: Invalid string identifier, element not found.")
          { }
          invalid_string_id(std::string s): record_exception("bit:Index: Invalid numeric identifier, element not found.")
          {
            m_exception_string = "bit:Index: No element with string identifier (" + s + ")";
          }
        }
      ;


    }
  }
}
#endif
