/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "field.h"

#include "record.h"

#include <iostream>

#include <string>
#include <sstream>
#include <ctype.h>

#define WHOLE_UNITS(__offset, __units) \
  ( ((__offset) > 0.0) ? ( (__offset)/__units+(((__offset)%__units)?1:0) ) : ( (__offset)/__units+(((__offset)%__units)?(-1):0)) )

namespace bit
{

  Field::Field ( std::string name, size_t length, size_t length_units, int offset, size_t offset_units, std::string description ) :
      m_length ( length ),
      m_length_units ( length_units ),
      m_offset ( offset ),
      m_offset_units ( offset_units ),
      m_description ( description ),
      m_encoding( ENCODING_NONE )
  {
    set_name ( name );
  }

  Field::pointer Field::create ( std::string name, size_t length, size_t length_units, int offset, size_t offset_units, std::string description )
  {
    return Field::pointer ( new Field ( name, length, length_units, offset, offset_units, description ) );
  }

  Field::~Field()
  { }

  size_t Field::length() const
  {
    return m_length;
  }

  size_t Field::length ( size_t units ) const
  {
    return this->FieldBase::length ( units );
  }

  void Field::set_length ( size_t length )
  {
    m_length = length;
  }

  size_t Field::length_units() const
  {
    return m_length_units;
  }

  void Field::set_length_units ( size_t units )
  {
    m_length_units = units;
  }

  int Field::offset() const
  {
    return m_offset;
  }

  int Field::offset ( size_t units ) const
  {
    return this->FieldBase::offset ( units );
  }

  void Field::set_offset ( int offset )
  {
    m_offset = offset;
  }

  size_t Field::offset_units() const
  {
    return m_offset_units;
  }

  void Field::set_offset_units ( size_t units )
  {
    m_offset_units = units;
  }

  std::string Field::name() const
  {
    return m_name;
  }

  std::string Field::name ( int depth ) const
  {
    return this->FieldBase::name ( depth );
  }


  void Field::set_name ( const std::string& name )
  {
    if ( m_name.find_first_of ( "[]." ) < m_name.size() )
      throw exception::name ( name );
    m_name = name;
  }

  std::string Field::description() const
  {
    return m_description;
  }

  void Field::set_description ( const std::string& description )
  {
    m_description = description;
  }

  const FieldType& Field::type() const
  {
    return m_type;
  }

  void Field::set_type ( const FieldType& type )
  {
    m_type = type;
  }

  Encoding Field::encoding() const
  {
    if ( m_parent && m_encoding == ENCODING_NONE )
      return m_parent->encoding();
    
    return m_encoding;
  }

  void Field::set_encoding( Encoding en )
  {
    m_encoding = en;
  }
  
  const std::string & Field::xml( )
  {
    std::ostringstream sout;
    sout << "<field name=\"" << m_name
    << "\" start=\"" << m_offset
    << "\" start_units=\"" << m_offset_units
    << "\" length=\"" << m_length
    << "\" length_units=\"" << m_length_units << "\"";
    if ( m_type.type() == TYPE_NONE )
      sout << "/>";
    else
      sout << ">" << m_type.xml() << "</field>";
    m_xml = sout.str();
    return m_xml;
  }

  FieldBase::pointer Field::clone()
  {
    Field::pointer field = Field::create ( m_name, m_length, m_length_units, m_offset, m_offset_units, m_description );
    field->m_type = m_type;
    return field;
  }

}
