/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "recordbase.h"

#define WHOLE_UNITS(__offset, __units) \
  ( ((__offset) > 0.0) ? ( (__offset)/__units+(((__offset)%__units)?1:0) ) : ( (__offset)/__units+(((__offset)%__units)?(-1):0)) )

namespace bit
  {

  RecordBase::RecordBase(std::string name, size_t length_units, int offset, size_t offset_units, std::string description):
      m_length_units(length_units),
      m_offset(offset),
      m_offset_units(offset_units),
      m_description(description),
      m_encoding( ENCODING_NONE )
  {
    set_name(name);
  }


  RecordBase::~RecordBase()
  {}

  size_t RecordBase::length_units() const
    {
      return m_length_units;
    }


  void RecordBase::set_length_units(size_t units)
  {
    m_length_units = units;
  }

  int RecordBase::offset() const
    {
      return m_offset;
    }

  int RecordBase::offset( size_t units ) const
    {
      return this->FieldBase::offset( units );
    }

  void RecordBase::set_offset(int offset)
  {
    m_offset = offset;
  }

  size_t RecordBase::offset_units() const
    {
      return m_offset_units;
    }

  void RecordBase::set_offset_units(size_t units)
  {
    m_offset_units = units;
  }

  std::string RecordBase::name() const
    {
      return m_name;
    }

  std::string RecordBase::name(int depth) const
    {
      return this->FieldBase::name(depth);
    }

  void RecordBase::set_name(const std::string& name)
  {
    if (m_name.find_first_of("[].") < m_name.size())
      throw exception::name(name);
    m_name = name;
  }

  std::string RecordBase::description() const
    {
      return m_description;
    }

  void RecordBase::set_description(const std::string& description)
  {
    m_description = description;
  }

  const FieldType& RecordBase::type() const
  {
    return m_type;
  }

  void RecordBase::set_type ( const FieldType& type )
  {
    m_type = type;
  }

  Encoding RecordBase::encoding() const
  {
    if ( m_parent && m_encoding == ENCODING_NONE )
      return m_parent->encoding();
    
    return m_encoding;
  }

  void RecordBase::set_encoding( Encoding en )
  {
    m_encoding = en;
  }
  


}
