/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "recordstorage.h"

namespace bit
  {

  RecordStorage::RecordStorage(FieldBase& record): m_record(&record)
  {}


  RecordStorage::~RecordStorage()
  {}

  FieldBase::iterator RecordStorage::begin()
  {
    return FieldBase::iterator( m_record, *(field_set.begin()) );
  }

  FieldBase::iterator RecordStorage::end()
  {
    return FieldBase::iterator( m_record, FieldBase::pointer() );
  }

  RecordStorage& RecordStorage::operator=(const RecordStorage& other) {
    field_set.clear();
    field_name_map.clear();
    FieldBase::pointer field;
    Fields::const_iterator i;

    for (i = other.field_set.begin(); i != other.field_set.end(); i++) {
      field = (*i)->clone();
      // TODO put in a proper throw
      if ( field->m_parent != NULL )
        throw;
      add_field(field);
    }

    return *this;
  }

  size_t RecordStorage::subfields()
  {
    return field_set.size();
  }

  FieldBase::pointer RecordStorage::subfield(const std::string& s) {
    size_t split = s.find_first_of(".[");

    if (split == std::string::npos )
    {
      FieldNameMap::iterator iter = field_name_map.find(s);
      if (iter == field_name_map.end() )
        return FieldBase::pointer();
      else
        return *(iter->second);
    }
    else
    {
      std::string field, remainder;
      field = s.substr(0, split);
      if ( s[split] == '.' )
        remainder = s.substr(split+1, s.size());
      else
        remainder = s.substr(split, s.size());
      FieldNameMap::iterator iter = field_name_map.find(field);
      if ( iter == field_name_map.end() )
        return FieldBase::pointer();
      else
      {
        FieldBase::pointer p = *(iter->second);
        return p->subfield(remainder);
      }
    }
  }

  FieldBase::pointer RecordStorage::subfield(size_t i) {
    if ( i >= field_set.size() )
      return FieldBase::pointer();

    Fields::iterator iter = field_set.begin();
    std::advance(iter, i);
    return *iter;
  }

  void RecordStorage::add_field(FieldBase::pointer field)
  {
    // TODO put in a proper throw
    if ( field->m_parent != NULL )
      throw;

    field->m_parent = m_record;

    Fields::iterator iter = field_set.insert(field).first;
    field_name_map[ field->name() ] = iter;
  }

  void RecordStorage::remove_field(const std::string& name)
  {
    FieldNameMap::iterator i = field_name_map.find(name);

    if ( i == field_name_map.end() )
      return;

    FieldBase::pointer field = *(i->second);
    // clear field parent
    field->m_parent = NULL;

    field_name_map.erase( field->name() );
    field_set.erase( field );
  }

  void RecordStorage::remove_field(FieldBase::pointer field)
  {
    if ( field_set.find(field) == field_set.end() )
      return;

    // clear field parent
    field->m_parent = NULL;

    field_name_map.erase( field->name() );
    field_set.erase( field );
  }

  FieldBase::pointer RecordStorage::previous_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {
    if ( ! current_field )
      return FieldBase::pointer();

    Fields::iterator iter = field_set.find(current_field);
    if ( iter == field_set.end() || iter == field_set.begin() )
      return FieldBase::pointer();

    return *(--iter);
  }

  FieldBase::pointer RecordStorage::next_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {

    if ( ! current_field )
      return FieldBase::pointer();

    Fields::iterator iter = field_set.find(current_field);
    if ( iter != field_set.end() )
      iter++;
    if ( iter == field_set.end() )
      return FieldBase::pointer();
    else
      return *iter;
  }

}
