/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the bit library.                                 *
 *                                                                         *
 *   The bit library is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The bit library is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#include "utility.h"

#include <cstring>
#include <endian.h>
#include <byteswap.h>

namespace bit {

# if __BYTE_ORDER == __BIG_ENDIAN

uint64_t be_to_host( uint64_t x) { return (x); }
int64_t be_to_host( int64_t x)   { return (x); }
uint32_t be_to_host( uint32_t x) { return (x); }
int32_t be_to_host( int32_t x)   { return (x); }
uint16_t be_to_host( uint16_t x) { return (x); }
int16_t be_to_host( int16_t x)   { return (x); }
uint8_t be_to_host( uint8_t x)   { return (x); }
int8_t be_to_host( int8_t x)     { return (x); }
long double be_to_host( long double x)     { return (x); }
double be_to_host( double x)     { return (x); }
float be_to_host( float x)       { return (x); }
uint64_t host_to_be( uint64_t x) { return (x); }
int64_t host_to_be( int64_t x)   { return (x); }
uint32_t host_to_be( uint32_t x) { return (x); }
int32_t host_to_be( int32_t x)   { return (x); }
uint16_t host_to_be( uint16_t x) { return (x); }
int16_t host_to_be( int16_t x)   { return (x); }
uint8_t host_to_be( uint8_t x)   { return (x); }
int8_t host_to_be( int8_t x)     { return (x); }
long double host_to_be( long double x)     { return (x); }
double host_to_be( double x)     { return (x); }
float host_to_be( float x)       { return (x); }
uint64_t le_to_host( uint64_t x) { return bswap_64 (x); }
int64_t le_to_host( int64_t x)   { return bswap_64 (x); }
uint32_t le_to_host( uint32_t x) { return bswap_32 (x); }
int32_t le_to_host( int32_t x)   { return bswap_32 (x); }
uint16_t le_to_host( uint16_t x) { return bswap_16 (x); }
int16_t le_to_host( int16_t x)   { return bswap_16 (x); }
uint8_t le_to_host( uint8_t x)   { return (x); }
int8_t le_to_host( int8_t x)     { return (x); }
long double le_to_host( long double x) {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u[1], 4 );
  memcpy( (uint8_t*)&x + 4, u[0], 4 );
  return x;
}
double le_to_host( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float le_to_host( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }
uint64_t host_to_le( uint64_t x) { return bswap_64 (x); }
int64_t host_to_le( int64_t x)   { return bswap_64 (x); }
uint32_t host_to_le( uint32_t x) { return bswap_32 (x); }
int32_t host_to_le( int32_t x)   { return bswap_32 (x); }
uint16_t host_to_le( uint16_t x) { return bswap_16 (x); }
int16_t host_to_le( int16_t x)   { return bswap_16 (x); }
uint8_t host_to_le( uint8_t x)   { return (x); }
int8_t host_to_le( int8_t x)     { return (x); }
long double host_to_le( long double x) {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u[1], 4 );
  memcpy( (uint8_t*)&x + 4, u[0], 4 );
  return x;
}
double host_to_le( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float host_to_le( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }

# elif __BYTE_ORDER == __LITTLE_ENDIAN

uint64_t be_to_host( uint64_t x) { return bswap_64 (x); }
int64_t be_to_host( int64_t x)   { return bswap_64 (x); }
uint32_t be_to_host( uint32_t x) { return bswap_32 (x); }
int32_t be_to_host( int32_t x)   { return bswap_32 (x); }
uint16_t be_to_host( uint16_t x) { return bswap_16 (x); }
int16_t be_to_host( int16_t x)   { return bswap_16 (x); }
uint8_t be_to_host( uint8_t x)   { return (x); }
int8_t be_to_host( int8_t x)     { return (x); }
long double be_to_host( long double x) {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u+1, 4 );
  memcpy( (uint8_t*)&x + 4, u, 4 );
  return x;
}
double be_to_host( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float be_to_host( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }
uint64_t host_to_be( uint64_t x) { return bswap_64 (x); }
int64_t host_to_be( int64_t x)   { return bswap_64 (x); }
uint32_t host_to_be( uint32_t x) { return bswap_32 (x); }
int32_t host_to_be( int32_t x)   { return bswap_32 (x); }
uint16_t host_to_be( uint16_t x) { return bswap_16 (x); }
int16_t host_to_be( int16_t x)   { return bswap_16 (x); }
uint8_t host_to_be( uint8_t x)   { return (x); }
int8_t host_to_be( int8_t x)     { return (x); }
long double host_to_be( long double x) {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u+1, 4 );
  memcpy( (uint8_t*)&x + 4, u, 4 );
  return x;
}
double host_to_be( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float host_to_be( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }
uint64_t le_to_host( uint64_t x) { return (x); }
int64_t le_to_host( int64_t x)   { return (x); }
uint32_t le_to_host( uint32_t x) { return (x); }
int32_t le_to_host( int32_t x)   { return (x); }
uint16_t le_to_host( uint16_t x) { return (x); }
int16_t le_to_host( int16_t x)   { return (x); }
uint8_t le_to_host( uint8_t x)   { return (x); }
int8_t le_to_host( int8_t x)     { return (x); }
long double le_to_host( long double x)     { return (x); }
double le_to_host( double x)     { return (x); }
float le_to_host( float x)       { return (x); }
uint64_t host_to_le( uint64_t x) { return (x); }
int64_t host_to_le( int64_t x)   { return (x); }
uint32_t host_to_le( uint32_t x) { return (x); }
int32_t host_to_le( int32_t x)   { return (x); }
uint16_t host_to_le( uint16_t x) { return (x); }
int16_t host_to_le( int16_t x)   { return (x); }
uint8_t host_to_le( uint8_t x)   { return (x); }
int8_t host_to_le( int8_t x)     { return (x); }
long double host_to_le( long double x)     { return (x); }
double host_to_le( double x)     { return (x); }
float host_to_le( float x)       { return (x); }

# endif

uint64_t net_to_host( uint64_t x) { return be_to_host(x); }
int64_t net_to_host( int64_t x) { return be_to_host(x); }
uint32_t net_to_host( uint32_t x) { return be_to_host(x); }
int32_t net_to_host( int32_t x) { return be_to_host(x); }
uint16_t net_to_host( uint16_t x) { return be_to_host(x); }
int16_t net_to_host( int16_t x) { return be_to_host(x); }
uint8_t net_to_host( uint8_t x) { return (x); }
int8_t net_to_host( int8_t x) { return (x); }

uint64_t host_to_net( uint64_t x) { return host_to_be(x); }
int64_t host_to_net( int64_t x) { return host_to_be(x); }
uint32_t host_to_net( uint32_t x) { return host_to_be(x); }
int32_t host_to_net( int32_t x) { return host_to_be(x); }
uint16_t host_to_net( uint16_t x) { return host_to_be(x); }
int16_t host_to_net( int16_t x) { return host_to_be(x); }
uint8_t host_to_net( uint8_t x) { return (x); }
int8_t host_to_net( int8_t x) { return (x); }


uint64_t le_to_be( uint64_t x) { return bswap_64 (x); }
int64_t le_to_be( int64_t x)   { return bswap_64 (x); }
uint32_t le_to_be( uint32_t x) { return bswap_32 (x); }
int32_t le_to_be( int32_t x)   { return bswap_32 (x); }
uint16_t le_to_be( uint16_t x) { return bswap_16 (x); }
int16_t le_to_be( int16_t x)   { return bswap_16 (x); }
uint8_t le_to_be( uint8_t x)   { return (x); }
int8_t le_to_be( int8_t x)     { return (x); }

uint64_t le_to_net( uint64_t x) { return bswap_64 (x); }
int64_t le_to_net( int64_t x)   { return bswap_64 (x); }
uint32_t le_to_net( uint32_t x) { return bswap_32 (x); }
int32_t le_to_net( int32_t x)   { return bswap_32 (x); }
uint16_t le_to_net( uint16_t x) { return bswap_16 (x); }
int16_t le_to_net( int16_t x)   { return bswap_16 (x); }
uint8_t le_to_net( uint8_t x)   { return (x); }
int8_t le_to_net( int8_t x)     { return (x); }

uint64_t be_to_le( uint64_t x) { return bswap_64 (x); }
int64_t be_to_le( int64_t x)   { return bswap_64 (x); }
uint32_t be_to_le( uint32_t x) { return bswap_32 (x); }
int32_t be_to_le( int32_t x)   { return bswap_32 (x); }
uint16_t be_to_le( uint16_t x) { return bswap_16 (x); }
int16_t be_to_le( int16_t x)   { return bswap_16 (x); }
uint8_t be_to_le( uint8_t x)   { return (x); }
int8_t be_to_le( int8_t x)     { return (x); }

uint64_t net_to_le( uint64_t x) { return bswap_64 (x); }
int64_t net_to_le( int64_t x)   { return bswap_64 (x); }
uint32_t net_to_le( uint32_t x) { return bswap_32 (x); }
int32_t net_to_le( int32_t x)   { return bswap_32 (x); }
uint16_t net_to_le( uint16_t x) { return bswap_16 (x); }
int16_t net_to_le( int16_t x)   { return bswap_16 (x); }
uint8_t net_to_le( uint8_t x)   { return (x); }
int8_t net_to_le( int8_t x)     { return (x); }

double net_to_host(double x) { return be_to_host( x ); }
float net_to_host(float x) { return be_to_host( x ); }
long double net_to_host(long double x) { return be_to_host( x ); }

double host_to_net( double x) { return host_to_be( x ); }
float host_to_net( float x) { return host_to_be( x ); }
long double host_to_net( long double x) { return host_to_be( x ); }

double le_to_be( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float le_to_be( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }
long double le_to_be( long double x)     {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u+1, 4 );
  memcpy( (uint8_t*)&x + 4, u, 4 );
  return x;
}

double le_to_net( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float le_to_net( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }
long double le_to_net( long double x)     {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u+1, 4 );
  memcpy( (uint8_t*)&x + 4, u, 4 );
  return x;
}

double be_to_le( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float be_to_le( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }
long double be_to_le( long double x)     {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u+1, 4 );
  memcpy( (uint8_t*)&x + 4, u, 4 );
  return x;
}

double net_to_le( double x)     { uint64_t u = bswap_64(*(uint64_t*)&x); return *((double*)&u); }
float net_to_le( float x)       { uint32_t u = bswap_32(*(uint32_t*)&x); return *((float*)&u); }
long double net_to_le( long double x)     {
  uint64_t u[2];
  memcpy( u, &x, 8 );
  u[0] = bswap_64 (u[0]);
  u[1] = bswap_64 (u[1]);
  memcpy( &x, u+1, 4 );
  memcpy( (uint8_t*)&x + 4, u, 4 );
  return x;
}

size_t octet_ceiling(size_t bits) { return bits/8 + (bits%8?1:0); }

size_t octet_floor(size_t bits) { return bits/8; }

void* starting_octet(void* p, size_t bits) {
  return static_cast<uint8_t*>(p) + octet_floor(bits);
}

void* ending_octet(void* p, size_t bits) {
  return static_cast<uint8_t*>(p) + octet_ceiling(bits);
}

size_t last_octet_upper_bits(size_t bits) { return bits%8; }

size_t last_octet_lower_bits(size_t bits) { return (8 - bits%8)%8; }

void* left_shift(void* m, size_t octets, size_t bits) {
  if (m == NULL)
    return m;

  if (bits >= octets * 8) {
    memset(m, 0x00, octets);
    return m;
  }

  uint8_t* mem = static_cast<uint8_t*>(m);
  size_t offset, length;

  if (bits%8 == 0)
    offset = octets - octet_ceiling(bits);
  else
    offset = octets - octet_ceiling(bits)+1;
  length = octets-offset;

  if ( bits >= 8) {
    // left shift all whole octets
    memmove( mem, starting_octet(mem, bits), octets-octet_floor(bits) );
    memset( mem + offset, 0x00, length);
  }

  uint8_t temp;
  uint8_t* p;
  size_t rbits = bits % 8; // remaining bits
  if (rbits) {
    for (p = mem; p < mem+octets-octet_floor(bits)-1; p++) {
      *p <<= rbits;
      temp = *(p+1);
      temp >>= 8-rbits;
      *p |= temp;
    }
    *p <<= rbits;
  }
  return mem;
}

void* right_shift(void* m, size_t octets, size_t bits) {
  if (m == NULL)
    return m;

  if (bits >= octets * 8) {
    memset(m, 0x00, octets);
    return m;
  }

  uint8_t* mem = static_cast<uint8_t*>(m);
  size_t offset, length;

//   if (bits%8 == 0)
    offset = octet_floor(bits);
//   else
//     offset = octets - octet_ceiling(bits)+1;
  length = octets-offset;

  if ( bits >= 8) {
    // right shift all whole octets
    memmove( mem+offset, mem, length );
    memset( mem, 0x00, offset);
  }

  uint8_t temp;
  uint8_t* p;
  size_t rbits = bits % 8; // remaining bits
  if (rbits) {
    for (p = mem+octets-1; p > mem+offset; p--) {
      *p >>= rbits;
      temp = *(p-1);
      temp <<= 8-rbits;
      *p |= temp;
    }
    *p >>= rbits;
  }
  return mem;
}

std::string hex_string( void* buffer, size_t buffer_size,
                        bool uppercase,
                        std::string separator,
                        std::string prefix,
                        std::string postfix
                      )
{
  if (buffer == NULL || buffer_size == 0)
    return std::string();

  std::string s = prefix;
  char hex[3];

  hex[2] = 0x00;
  int16_t char_offset;

  if (uppercase)
    char_offset = 'A'-'0'-10;
  else
    char_offset = 'a'-'0'-10;

  for (uint8_t* p = (uint8_t*)buffer; p < (uint8_t*)buffer + buffer_size; p++)
  {
    if ( p != (uint8_t*)buffer )
      s += separator;
    hex[0] = '0' + (*p >> 4);
    hex[1] = '0' + (*p & 0x0F);
    if (hex[0] > '9')
      hex[0] = hex[0] + char_offset;
    if (hex[1] > '9')
      hex[1] = hex[1] + char_offset;

    s += hex;
  }
  s += postfix;
  return s;
}

std::string binary_string( void* buffer, size_t buffer_size,
                           std::string separator,
                           size_t separator_digits,
                           std::string prefix,
                        std::string postfix
                      )
{
  if (buffer == NULL || buffer_size == 0)
    return std::string();

  std::string s = prefix;
  uint8_t mask;

  for (uint8_t* p = (uint8_t*)buffer; p < (uint8_t*)buffer + buffer_size; p++)
  {
    mask = 0x80;
    for (int i=0; i<8; i++) {
      if (mask & *p)
        s += '1';
      else
        s += '0';
      mask >>= 1;
      if ( separator_digits > 0 && (i+1)%separator_digits == 0 && ! (p == (uint8_t*)buffer+buffer_size-1 && i == 7) )
        s += separator;
    }
  }
  s += postfix;
  return s;
}



}
